//$fn = 30;

//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////
//Library for involute spur, worm and rack gears
//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////

/*
Author:		Dr Jörg Janssen
date:		29. October 2018
Version:	2.3
license:	Creative Commons - Attribution, Non Commercial, Share Alike

Modified by John Mcphee
Date:       05 May 2019
Version     2.3.1
Modifications
1)  Translated to english
2)  Planetary spur gear module added
3)  Planetary bore size split into sun and planet variables
4)  Rack and gear thickness split for rack and spur gear
5)  Missing teeth variable added
6)  Bore for 28BYJ-48 type motor drive added    ????? herring bone_bevel_gear
                                                ??????herring_bone_bevel_gear_pair???? 
                                                ?????spiral_bevel_gear????
7)  Fixing holes to rack added                  ???????bolt size lookup tables to fill




Permitted modules according to DIN 780:

0.05 0.06 0.08 0.10 0.12 0.16
0.20 0.25 0.3  0.4  0.5  0.6
0.7  0.8  0.9  1    1.25 1.5
2    2.5  3    4    5    6
8    10   12   16   20   25
32   40   50   60

*/


//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////
//Examples of each module are commented out below
//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////


//  gear_rack (modulus=1, rack_length=100, pitch_line_height=10, rack_thickness=10, pressure_angle=20, helix_angle=20,fixing_centre=44,fixing_height_from_base=5,metric_cap_head=4);

//  spur_gear (modulus=1, number_of_teeth=30,missing_teeth=0, gear_thickness=8, bore_size=6, bore_width_of_flats=3, pressure_angle=20, helix_angle=20, optimized=true);

//  gear_rack_and_spur_gear (modulus=1, rack_length=100, number_of_teeth_on_gear=60,missing_teeth=3, pitch_line_height=10, bore_size_on_gear=5,bore_width_of_flats=3,rack_thickness=10, gear_thickness=20, pressure_angle=20, helix_angle=0, built_together=true, optimized=true,fixing_centre=30,fixing_height_from_base=4,metric_cap_head=3);

//  spur_ring_gear (modulus=1, number_of_teeth=30,missing_teeth=0, gear_thickness=5, ring_edge_width=3, pressure_angle=20, helix_angle=20);

//  herring_bone_spur_gear (modulus=1, number_of_teeth=30,missing_teeth=4, gear_thickness=6, bore_size=5,bore_width_of_flats=3, pressure_angle=20, helix_angle=30, optimized=true);

//  herring_bone_ring_gear (modulus=1, number_of_teeth=30,missing_teeth=5, gear_thickness=5, ring_edge_width=3, pressure_angle=20, helix_angle=30);

//  planetary_spur_gear(modulus=1, number_of_teeth_sun=40,missing_teeth_sun=0, number_of_teeth_planet=10,missing_teeth_planet=0,missing_teeth_ring=0, number_of_planets=6, gear_thickness=5, ring_edge_width=3, sun_bore_size=6,planet_bore_size=3,bore_width_of_flats=4, pressure_angle=20, helix_angle=30, built_together=true, optimized=true);

//  planetary_herring_bone_gear(modulus=1, number_of_teeth_sun=20,missing_teeth_sun=2, number_of_teeth_planet=10,missing_teeth_planet=1,missing_teeth_ring=7, number_of_planets=6, gear_thickness=5, ring_edge_width=3, sun_bore_size=5,planet_bore_size=2,bore_width_of_flats=3, pressure_angle=20, helix_angle=30, built_together=true, optimized=true);

//  bevel_gear(modulus=1, number_of_teeth=30,missing_teeth=5, bevel_angle=45, tooth_width=5, bore_size=5, bore_width_of_flats=3, pressure_angle=20, helix_angle=30);

//  herring_bone_bevel_gear(modulus=1, number_of_teeth=30,missing_teeth=5, bevel_angle=45, tooth_width=5, bore_size=6, bore_width_of_flats=5,pressure_angle=20, helix_angle=30);

//  bevel_gear_pair(modulus=1, number_of_teeth_on_gear=30,missing_teeth_gear=5, number_of_teeth_pinion=12,missing_teeth_pinion=2, angle_of_axis=100, tooth_width=5, bore_size_on_gear=7,bore_width_of_flats_gear=5 ,bore_size_pinion=5,bore_width_of_flats_pinion=3, pressure_angle=20, helix_angle=20, built_together=true);

//  herring_bone_bevel_gear_pair(modulus=1, number_of_teeth_on_gear=30,missing_teeth_gear=5, number_of_teeth_pinion=10,missing_teeth_pinion=2, angle_of_axis=90, tooth_width=5, bore_size_on_gear=7,bore_width_of_flats_gear=5, bore_size_pinion=5,bore_width_of_flats_pinion=3, pressure_angle = 20, helix_angle=30, built_together=false);

//  spiral_bevel_gear(modulus=1, number_of_teeth=20,missing_teeth=5, bevel_angle=45, tooth_width=5, bore_size=7,bore_width_of_flats=5, pressure_angle = 20, helix_angle=30);

//  worm(modulus=2, number_of_starts=2, worm_length=25, bore_size=7,bore_width_of_flats=5, pressure_angle=20, gradient_angle=10,unsplit_worm_print=false);

//  worm_and_spur_gear(modulus=2, number_of_teeth=25,missing_teeth=0, number_of_starts=2, width=8, worm_length=30, bore_size_worm=4, bore_width_of_flats_worm=2, bore_size_gear=6, bore_width_of_flats_gear=7, pressure_angle=20, gradient_angle=10, optimized=true, built_together=true,unsplit_worm_print=true);

//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////


// General variables
pi = 3.14159;       // pi, no explanation needed
rad = 57.29578;     // radians to degrees ratio
clearance = 0.05;	// Play between teeth

//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////

//functions used by gear modules

/*	Converts Radian to degrees */
function rad_to_deg(pressure_angle) = pressure_angle*rad;

/*	Converts degrees to Radian */
function deg_to_rad(pressure_angle) = pressure_angle/rad;

/*	Converts 2D polar coordinates to Cartesianm
    Format: radius, phi; phi = angle to x-axis on xy plane */
function polar_to_cartesian(polvect) = [
	polvect[0]*cos(polvect[1]),  
	polvect[0]*sin(polvect[1])
];

/*	Involutes function:
    Returns the polar coordinates of a circle involute
    r = Radius of the base circle
    rho = Rolling angle in degrees */
function ev(r,rho) = [
	r/cos(rho),
	rad_to_deg(tan(rho)-deg_to_rad(rho))
];

/*  curve inverlute function
     Returns the azimuth angle of a curve
     theta0 = polar angle of the cone, at the cutting edge of the large ball unrolls the involute
     theta = polar angle for which the azimuth angle of the involute is to be calculated */
function curve_inverlute(theta0,theta) = 1/sin(theta0)*acos(cos(theta)/cos(theta0))-acos(tan(theta0)/tan(theta));

/*  Converts spherical coordinates to Cartesian
     Format: radius, theta, phi; theta = angle to z-axis, phi = angle to x-axis on xy plane */
function radius_to_cartesian(vect) = [
	vect[0]*sin(vect[1])*cos(vect[2]),  
	vect[0]*sin(vect[1])*sin(vect[2]),
	vect[0]*cos(vect[1])
];

/*	Check if a number is even
	= 1, if the number is even
	= 0, if the number is not even */
function is_number_even(test_num) =
	(test_num == floor(test_num/2)*2) ? 1 : 0;

/*	greatest common divisor
	according to Euclidean algorithm.
	Sorting: a must be greater than b */
function ggt(a,b) = 
	a%b == 0 ? b : ggt(b,a%b);

/*	Polar function with polar angle and two variables */
function spiral(a, r0, phi) =
	a*phi + r0; 

/*	Copy and rotate a body */
module copy(vect, number, distance, angle){
	for(i = [0:number-1]){
		translate(v=vect*distance*i)
			rotate(a=i*angle, v = [0,0,1])
				children(0);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////


/*  gear_rack
    modulus = pitch diameter (infinity in case of rack) divided by number of teeth
    rack_length = Length of gear rack
    pitch_line_height = Height of the gear rack to the pitch line
    rack_thickness = thickness of the rack
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the gear_racks transverse axis; 0 ° = spur toothing
    fixing_centre= distance between fixing screws, will generate as many as fits
    fixing_height_from_base=centre height of fixing from base of rack
    metric_cap_head=metric cap head fixing thread size
*/

    
module gear_rack(   modulus,
                    rack_length,
                    pitch_line_height, 
                    rack_thickness, 
                    pressure_angle, 
                    helix_angle,
                    fixing_centre,
                    fixing_height_from_base,
                    metric_cap_head) 
{

	// Dimensions calculations
    
	modulus=modulus*(1-clearance);
	tip_clearance = modulus / 6;						// tip clearance
	gear_skew_angle = modulus/cos(helix_angle);			// Skew angle of gear in x direction
    skew_linear_displacement = rack_thickness*tan(helix_angle);	// Displacement of the upper side in the x-direction by helix angle
	flank_height = 2*gear_skew_angle*tan(pressure_angle)+tip_clearance*tan(pressure_angle);		// Flank width
	face_height = pi*gear_skew_angle/2-2*gear_skew_angle*tan(pressure_angle);			// Face width
	
	num_rack_teeth = ceil((rack_length+abs(2*skew_linear_displacement))/(pi*gear_skew_angle));					// number of teeth
    
    
    //my bit jmc
	
    number_of_fixings=floor((rack_length-ThreadToCapHeadDiameter(metric_cap_head)-4)/fixing_centre)+1;// calculate number of fixings
    echo ("number_of_fixings",number_of_fixings);
    
    difference()
    {  
        union()
        {//draw shape
            
    //end my bit jmc
    
    // generate shape
    
            translate([-pi*gear_skew_angle*(num_rack_teeth-1)/2-flank_height-face_height/2,-modulus,0])
            {
                intersection()          // Creates a prism that fits into a cuboid geometry
                    {										
                    copy([1,0,0], num_rack_teeth, pi*gear_skew_angle, 0)
                        {
                        polyhedron(
                                    points=[[0,-tip_clearance,0],
                                    [flank_height,2*modulus,0], 
                                    [flank_height+face_height,2*modulus,0], 
                                    [2*flank_height+face_height,-tip_clearance,0], 
                                    [pi*gear_skew_angle,-tip_clearance,0], 
                                    [pi*gear_skew_angle,modulus-pitch_line_height,0], 
                                    [0,modulus-pitch_line_height,0],	// bottom
                                    [0+skew_linear_displacement,-tip_clearance,rack_thickness], 
                                    [flank_height+skew_linear_displacement,2*modulus,rack_thickness], 
                                    [flank_height+face_height+skew_linear_displacement,2*modulus,rack_thickness], 
                                    [2*flank_height+face_height+skew_linear_displacement,-tip_clearance,rack_thickness], 
                                    [pi*gear_skew_angle+skew_linear_displacement,-tip_clearance,rack_thickness], 
                                    [pi*gear_skew_angle+skew_linear_displacement,modulus-pitch_line_height,rack_thickness], 
                                    [0+skew_linear_displacement,modulus-pitch_line_height,rack_thickness]],	// top
                
                                    faces=[ [6,5,4,3,2,1,0],						// bottom
                                            [1,8,7,0],
                                            [9,8,1,2],
                                            [10,9,2,3],
                                            [11,10,3,4],
                                            [12,11,4,5],
                                            [13,12,5,6],
                                            [7,13,6,0],
                                            [7,8,9,10,11,12,13],					// top
                                            ]);
                        };
                        translate([abs(skew_linear_displacement),-pitch_line_height+modulus-0.5,-0.5]){
                        cube([rack_length,pitch_line_height+modulus+1,rack_thickness+1]);			        // Cuboid, which includes the volume of gear rack
			}	
                    };
            };


            }
            
            //my bit again jmc
            //start of my remove fixings

            //fixing spacing
            //fix point of first fixing
            translate([ -(fixing_centre*(number_of_fixings-1)/2 ),0,0])
            {           
                //locate all the fixing holes
                for(i = [0:number_of_fixings-1])
                {
                    translate([fixing_centre*(i),- pitch_line_height+fixing_height_from_base,0])   
                    {
                        //fixing details
                        //screw clearance hole    
                        //translate([0,0,0])    
                        cylinder(  rack_thickness, d1=ThreadToClearanceHole(metric_cap_head),  d2=ThreadToClearanceHole(metric_cap_head)  );
                        //cap head cylinder    
                        translate([0,0,rack_thickness-ThreadToCapHeadDepth(metric_cap_head)])    
                        cylinder(  ThreadToCapHeadDepth(metric_cap_head)+.01, d1=ThreadToCapHeadDiameter(metric_cap_head),  d2=ThreadToCapHeadDiameter(metric_cap_head)  );
                    }
                }
            }
            //end my bit again jmc
            
    }
}


/*  spur_gear
    modulus =  Pitch diameter divided by number of teeth
    number_of_teeth = Number of teeth on gear
    missing_teeth = number of teeth not included on gear
    gear_thickness = Thickness of gear
    bore_size = Diameter of the center bore
    pressure_angle = Pressure_angle, Standard value = 20 ° according to DIN 867. 
                                     Should not be greater than 45 °.
    helix_angle = Helix angle to the axis of rotation; 
                  0 ° = spur toothing
	optimized = Holes for material / weight savings.
                Create surface enlargement if geometry allows */

module spur_gear(   modulus,
                    number_of_teeth,
                    missing_teeth,
                    gear_thickness, 
                    bore_size,
                    bore_width_of_flats,
                    pressure_angle = 20, 
                    helix_angle = 0, 
                    optimized = true) 
{

	// Dimensions calculations
	pitch_circle_dia = modulus * number_of_teeth;			        // Pitch diameter
	pitch_circle_radius = pitch_circle_dia / 2;						// Pitch radius
	alpha_angle = atan(tan(pressure_angle)/cos(helix_angle));       // Helix angle in the frontal section
	base_circle_dia = pitch_circle_dia * cos(alpha_angle);			// Base circle diameter
	base_circle_radius = base_circle_dia / 2;						// Base circle radius
	tip_circle_dia = (modulus <1)? pitch_circle_dia + modulus * 2.2 : pitch_circle_dia + modulus * 2;			// Tip diameter according to DIN 58400 or DIN 867
	tip_circle_radius = tip_circle_dia / 2;								// tip circle radius
	tip_clearance =  (number_of_teeth <3)? 0 : modulus/6;			// tip clearance
	root_circle_dia = pitch_circle_dia - 2 * (modulus + tip_clearance);		// root circle diameter
	root_circle_radius = root_circle_dia / 2;													        // root circle radius
	max_roll_angle = acos(base_circle_radius/tip_circle_radius);	   // maximum roll angle;
                                                               // Involute begins on the base circle and ends at the tip circle
	roll_angle_at_pitch_circle = acos(base_circle_radius/pitch_circle_radius);												// Rolling angle at the pitch circle;
																	// Involute begins on the base circle and ends at the tip circle
	angle_to_involute_point = rad_to_deg(tan(roll_angle_at_pitch_circle)-deg_to_rad(roll_angle_at_pitch_circle));				// Angle to point of involute on pitch circle
	gear_skew_angle = rad*gear_thickness/(pitch_circle_radius*tan(90-helix_angle));   // Twist angle for linear extrusion
	step = max_roll_angle/16;									// Involute is divided into 16 pieces
	circ_pitch = 360/number_of_teeth;					// circular pitch angle
	
	r_weight_save_hole = (2*root_circle_radius - bore_size)/8;	// Radius of holes for material / weight savings
	weight_save_rad_from_centre = bore_size/2+2*r_weight_save_hole;	// Distance of the axes of the holes from the main axis
	num_weight_save_holes = floor(2*pi*weight_save_rad_from_centre/(3*r_weight_save_hole));	// Number of holes for material / weight savings
	
	optimized = (optimized && pitch_circle_radius >= gear_thickness*1.5 && pitch_circle_dia > 2*bore_size);	// is optimization useful?

	// generate shape
	union(){
		rotate([0,0,-angle_to_involute_point-90*(1-clearance)/number_of_teeth]){			// Center the tooth on the x-axis;
																		// makes alignment with other wheels easier

			linear_extrude(height = gear_thickness, twist = gear_skew_angle){
				difference(){
					union(){
						tooth_width = (180*(1-clearance))/number_of_teeth+2*angle_to_involute_point;
						circle(root_circle_radius);										    // root circle
                        //modified to remove "missing_teeth"
						for (rot = [0:circ_pitch:360-((missing_teeth+1)*circ_pitch)]){
							rotate (rot){								    // "number_of_teeth-times" copy and rotate
								polygon(concat(							    // Zahn
									[[0,0]],    // Tooth segment begins and ends in the origin
									[for (rho = [0:step:max_roll_angle]) // from zero rad_to_deg (base circle)
                                                                            // to maximum involute angle (top circle)
                                        polar_to_cartesian(ev(base_circle_radius,rho))],	// First involute flank

									[polar_to_cartesian(ev(base_circle_radius,max_roll_angle))],	// Point of involute on top circle
									[for (rho = [max_roll_angle:-step:0])	    // of maximum involute angle (top circle)
                                                                            // bis null rad_to_deg (Grundkreis)
										polar_to_cartesian([ev(base_circle_radius,rho)[0], tooth_width-ev(base_circle_radius,rho)[1]])]
                                                                            // Second involute flank
                                                                            // (180*(1-clearance)) statt 180 rad_to_deg,
                                                                            // to allow clearance on the flanks
									)
								);
							}
						}
					}			
					circle(r = weight_save_rad_from_centre+r_weight_save_hole*1.49);							// "bore_size"
				}
			}
		}
		// with material savings
		if (optimized) 
        {
			linear_extrude(height = gear_thickness){
				difference()
                {
						circle(r = (bore_size+r_weight_save_hole)/2);
//						circle(r = bore_size/2);							// centre bore hole ( this was original code )
                    
                        // the next bit added to put flats on side of bore
                        // if no flats required make flats parameter bigger than bore parameter                

                        difference()
                        { 
                        //draw a 2D circle on centre axis
                        circle(d=bore_size);        
                        //draw a square and deduct from circle to blank one side
                        translate([-bore_width_of_flats*1.5,-bore_size/2]) 
                        square([bore_width_of_flats,bore_size]);
                        //draw a square and deduct from circle to blank other side
                        translate([bore_width_of_flats/2,-bore_size/2]) 
                        square([bore_width_of_flats,bore_size]);
                        }
                        // end of flats coding
				}
			}
			linear_extrude(height = (gear_thickness-r_weight_save_hole/2 < gear_thickness*2/3) ? gear_thickness*2/3 : gear_thickness-r_weight_save_hole/2){
				difference(){
					circle(r=weight_save_rad_from_centre+r_weight_save_hole*1.51);
					union(){
						circle(r=(bore_size+r_weight_save_hole)/2);
						for (i = [0:1:num_weight_save_holes]){
							translate(radius_to_cartesian([weight_save_rad_from_centre,90,i*360/num_weight_save_holes]))
								circle(r = r_weight_save_hole);
						}
					}
				}
			}
		}
		// without material savings
		else 
        {
			linear_extrude(height = gear_thickness)
            {
				difference()
                {
					circle(r = weight_save_rad_from_centre+r_weight_save_hole*1.51);
//			        circle(r = bore_size/2);            // centre bore hole ( this was original code )
                    
                        // the next bit added to put flats on side of bore
                        // if no flats required make flats parameter bigger than bore parameter                

                    difference()
                    { 
                    //draw a 2D circle on centre axis
                    circle(d=bore_size);        
                    //draw a square and deduct from circle to blank one side
                    translate([-bore_width_of_flats*1.5,-bore_size/2]) 
                    square([bore_width_of_flats,bore_size]);
                    //draw a square and deduct from circle to blank other side
                    translate([bore_width_of_flats/2,-bore_size/2]) 
                    square([bore_width_of_flats,bore_size]);
                    }
                    // end of flats coding
                    
				}
			}
		}
	}
}

/*  herring_bone_spur_gear; uses the module "spur_gear"
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth = Number of gear teeth
    missing_teeth =
    gear_thickness = thickness of gear
    bore_size = Diameter of the center bore
    bore_width_of_flats= flats on bore for 28BYJ style motors
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the rotation axis, default value = 0 ° (straight toothing)
	optimized = Holes for material / weight savings */

module herring_bone_spur_gear(   modulus, 
                            number_of_teeth,
                            missing_teeth,
                            gear_thickness, 
                            bore_size,
                            bore_width_of_flats,
                            pressure_angle = 20, 
                            helix_angle=0, 
                            optimized)
{

	gear_thickness = gear_thickness/2;
	pitch_circle_dia = modulus * number_of_teeth;									// Pitch diameter
	pitch_circle_radius = pitch_circle_dia / 2;														// Pitch radius
	tip_clearance =  (number_of_teeth <3)? 0 : modulus/6;						// TIP clearance

	root_circle_dia = pitch_circle_dia - 2 * (modulus + tip_clearance);										// root diameter
	root_circle_radius = root_circle_dia / 2;													// root circle radius
	r_weight_save_hole = (2*root_circle_radius - bore_size)/8;									// Radius of holes for material / weight savings
	weight_save_rad_from_centre = bore_size/2+2*r_weight_save_hole;										// Distance of the axes of the holes from the main axis
	num_weight_save_holes = floor(2*pi*weight_save_rad_from_centre/(3*r_weight_save_hole));								// Number of holes for material / weight savings
	
	optimized = (optimized && pitch_circle_radius >= gear_thickness*3 && pitch_circle_dia > 2*bore_size);		// is optimization useful?

	translate([0,0,gear_thickness]){
		union(){
            
 			spur_gear(modulus, number_of_teeth,missing_teeth, gear_thickness,bore_size ,bore_width_of_flats, pressure_angle, helix_angle, optimized);		// bottom half
			mirror([0,0,1]){
			spur_gear(modulus, number_of_teeth,missing_teeth, gear_thickness, bore_size,bore_width_of_flats, pressure_angle, helix_angle, optimized);	// upper half           
            
//			spur_gear(modulus, number_of_teeth, gear_thickness, 2*(weight_save_rad_from_centre+r_weight_save_hole*1.49),bore_width_of_flats, pressure_angle, helix_angle, false);		// bottom half
//			mirror([0,0,1]){
//				spur_gear(modulus, number_of_teeth, gear_thickness, 2*(weight_save_rad_from_centre+r_weight_save_hole*1.49),bore_width_of_flats, pressure_angle, helix_angle, false);	// upper half
			
            
            
            }
		}
	}
    
 /*   
	// with material savings
	if (optimized) {
		linear_extrude(height = gear_thickness*2){
			difference(){
					circle(r = (bore_size+r_weight_save_hole)/2);
					circle(r = bore_size/2);							// bore_size
				}
			}
		linear_extrude(height = (2*gear_thickness-r_weight_save_hole/2 < 1.33*gear_thickness) ? 1.33*gear_thickness : 2*gear_thickness-r_weight_save_hole/2){ //gear_thickness*4/3
			difference(){
				circle(r=weight_save_rad_from_centre+r_weight_save_hole*1.51);
				union(){
					circle(r=(bore_size+r_weight_save_hole)/2);
					for (i = [0:1:num_weight_save_holes]){
						translate(radius_to_cartesian([weight_save_rad_from_centre,90,i*360/num_weight_save_holes]))
							circle(r = r_weight_save_hole);
					}
				}
			}
		}
	}
	// without material savings
	else {
		linear_extrude(height = gear_thickness*2){
			difference(){
				circle(r = weight_save_rad_from_centre+r_weight_save_hole*1.51);
				circle(r = bore_size/2);
			}
		}
	}
    
 */   
    
}

/*	gear_rack_and_spur_gear
    modulus =  pitch diameter divided by number of teeth 
    rack_length = length of the gear rack
    number_of_teeth_on_gear = Number of gear teeth
    missing_teeth = number of missing teeth on gear
	pitch_line_height = Höhe der gear_rack bis zur Wälzgeraden
    bore_size_on_gear = Diameter of the center bore of the spur gears
    bore_width_of_flats=flats on bore for  28BYJ type motors,
    rack_thickness= thickness of rack,
	gear_thickness = thickness of the gear,
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the rotation axis, default value = 0 ° (straight toothing)
    fixing_centre = fixing
    fixing_height_from_base =
    metric_cap_head =
*/

module gear_rack_and_spur_gear (modulus, 
                                rack_length, 
                                number_of_teeth_on_gear,
                                missing_teeth,
                                pitch_line_height, 
                                bore_size_on_gear,
                                bore_width_of_flats,
                                rack_thickness,
                                gear_thickness, 
                                pressure_angle, 
                                helix_angle, 
                                built_together, 
                                optimized,
                                fixing_centre,
                                fixing_height_from_base,
                                metric_cap_head) 
{

	distance = built_together? modulus*number_of_teeth_on_gear/2 : modulus*number_of_teeth_on_gear;

	gear_rack(modulus, rack_length, pitch_line_height, rack_thickness, pressure_angle, -helix_angle,fixing_centre, fixing_height_from_base, metric_cap_head);
	translate([0,distance,0])
		rotate(a=360/number_of_teeth_on_gear)
			spur_gear (modulus, number_of_teeth_on_gear,missing_teeth, gear_thickness, bore_size_on_gear,bore_width_of_flats, pressure_angle, helix_angle, optimized);
}

/*	spur_ring_gear
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth = Number of gear teeth
    missing_teeth = 
    gear_thickness = thickness of gear
	ring_edge_width = width of the edge from root circle
    bore_size = Diameter of the center bore
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the rotation axis, default value = 0 ° (straight toothing) */

module spur_ring_gear(  modulus, 
                        number_of_teeth,
                        missing_teeth,
                        gear_thickness, 
                        ring_edge_width, 
                        pressure_angle = 20, 
                        helix_angle = 0) 
{

	// Dimensions calculations
	ha = (number_of_teeth >= 20) ? 0.02 * atan((number_of_teeth/15)/pi) : 0.6;	// Shortening factor of tooth tip height
	pitch_circle_dia = modulus * number_of_teeth;									// Pitch diameter
	pitch_circle_radius = pitch_circle_dia / 2;														// Pitch radius
	alpha_angle = atan(tan(pressure_angle)/cos(helix_angle));       // Helix angle in the frontal section
	base_circle_dia = pitch_circle_dia * cos(alpha_angle);							// Base circle diameter
	base_circle_radius = base_circle_dia / 2;										// Base circle radius
	tip_clearance = modulus / 6;												// tip clearance
	tip_circle_dia = (modulus <1)? pitch_circle_dia + (modulus+tip_clearance) * 2.2 : pitch_circle_dia + (modulus+tip_clearance) * 2;	// Tip diameter
	tip_circle_radius = tip_circle_dia / 2;					// Tip circle radius
	root_circle_dia = pitch_circle_dia - 2 * modulus * ha;										// root diameter
	root_circle_radius = root_circle_dia / 2;													// root circle radius
	max_roll_angle = acos(base_circle_radius/tip_circle_radius);											// maximum involute angle;
																	// Involute begins on the base circle and ends at the tip circle
	roll_angle_at_pitch_circle = acos(base_circle_radius/pitch_circle_radius);												// Involute angle at pitch circle;
																	// Involute begins on the base circle and ends at the tip circle
	angle_to_involute_point = rad_to_deg(tan(roll_angle_at_pitch_circle)-deg_to_rad(roll_angle_at_pitch_circle));				// Angle to point of involute on pitch circle
	gear_skew_angle = rad*gear_thickness/(pitch_circle_radius*tan(90-helix_angle));				        // Twist angle for linear extrusion
	step = max_roll_angle/16;											// Involute is divided into 16 pieces
	circ_pitch = 360/number_of_teeth;								// pitch angle

	// generate shape
	rotate([0,0,-angle_to_involute_point-90*(1+clearance)/number_of_teeth])			// Center the tooth on the x-axis;
																	// makes alignment with other wheels easier
	linear_extrude(height = gear_thickness, twist = gear_skew_angle){
		difference(){
			circle(r = tip_circle_radius + ring_edge_width);							// Außenkreis
			union(){
				tooth_width = (180*(1+clearance))/number_of_teeth+2*angle_to_involute_point;
				circle(root_circle_radius);											// root circle
				for (rot = [0:circ_pitch:360-((missing_teeth+1)*circ_pitch)]){
					rotate (rot) {									// "number_of_teeth times" copy and rotate
						polygon( concat(
							[[0,0]],
							[for (rho = [0:step:max_roll_angle])			// from zero rad_to_deg (base circle)
																	// to maximum involute angle (top circle)
								polar_to_cartesian(ev(base_circle_radius,rho))],
							[polar_to_cartesian(ev(base_circle_radius,max_roll_angle))],
							[for (rho = [max_roll_angle:-step:0])		// of maximum involute angle (top circle)
																	// to zero rad_to_deg (base circle)
								polar_to_cartesian([ev(base_circle_radius,rho)[0], tooth_width-ev(base_circle_radius,rho)[1]])]
																	// (180 * (1 + clearance)) instead of 180,
																	// to allow clearance on the flanks
							)
						);
					}
				}
			}
		}
	}

	echo("Outer diameter ring_gear = ", 2*(tip_circle_radius + ring_edge_width));
	
}

/*  herring_bone_ring_gear; uses the modulue "ring_gear"
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth = Number of gear teeth
    missing_teeth =
    gear_thickness = thickness of gear
    bore_size = Diameter of the center bore
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the rotation axis, default value = 0 ° (straight toothing) */

module herring_bone_ring_gear(  modulus, 
                                number_of_teeth,
                                missing_teeth,
                                gear_thickness, 
                                ring_edge_width, 
                                pressure_angle = 20, 
                                helix_angle = 0) 
{

	gear_thickness = gear_thickness / 2;
	translate([0,0,gear_thickness])
		union(){
		spur_ring_gear(modulus, number_of_teeth,missing_teeth, gear_thickness, ring_edge_width, pressure_angle, helix_angle);		// bottom half
		mirror([0,0,1])
			spur_ring_gear(modulus, number_of_teeth,missing_teeth, gear_thickness, ring_edge_width, pressure_angle, helix_angle);	// upper half
            
	}
}


 //planetary_spur_gear(modulus=1, number_of_teeth_sun=20,missing_teeth_sun, number_of_teeth_planet=10,missing_teeth_planet, number_of_planets=6, gear_thickness=5, ring_edge_width=3, sun_bore_size=5,planet_bore_size=2,bore_width_of_flats=3, pressure_angle=20, helix_angle=30, built_together=true, optimized=true);



/*	planetary_spur_gear; uses the modules "spur_gear" and "spur_ring_gear"
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth_sun = Number of teeth of sun gear
    missing_teeth_sun =
    number_of_teeth_planet = Number of teeth of a planetary gear
    missing_teeth_planet =
    missing_teeth_ring = 
    Number of planet gears=number of planet gears, If null, the function will calculate the minimum number.
    gear_thickness = thickness of gear
	ring_edge_width = width of the edge from root circle
    sun_bore_size = Diameter of the sun gear center bore
    planet_bore_size = Diameter of the planet gear center bore
    bore_width_of_flats=
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the rotation axis, default value = 0 ° (straight toothing)
    built_together = Components assembled for construction or apart for 3D printing
	optimized = Holes for material / weight savings bzw. Create surface enlargement if geometry allows
		*/

module planetary_spur_gear( modulus, 
                            number_of_teeth_sun,
                            missing_teeth_sun, 
                            number_of_teeth_planet,
                            missing_teeth_planet,
                            missing_teeth_ring,
                            number_of_planets, 
                            gear_thickness, 
                            ring_edge_width,
                            sun_bore_size,
                            planet_bore_size,
                            bore_width_of_flats,
                            pressure_angle=20, 
                            helix_angle=0, 
                            built_together=true, 
                            optimized=true)
{

	// Dimensions calculations
	d_sun = modulus*number_of_teeth_sun;				// Pitch circle diameter sun
	d_planet = modulus*number_of_teeth_planet;			// Pitch circle planet
	center_distance = modulus*(number_of_teeth_sun +  number_of_teeth_planet) / 2;		// Distance from sun gear / ring_gear axis
	number_of_teeth_ring_gear = number_of_teeth_sun + 2*number_of_teeth_planet;				// Number of teeth of the ring gear
    d_ring_gear = modulus*number_of_teeth_ring_gear;    // Pitch diameter ring gear

	is_even = is_number_even(number_of_teeth_planet);								// Does the sun gear have to be turned?
		
	n_max = floor(180/asin(modulus*(number_of_teeth_planet)/(modulus*(number_of_teeth_sun +  number_of_teeth_planet))));
																		// Number of planet gears: at most as many as without
																		// Overlap possible

	// generate shape
	rotate([0,0,180/number_of_teeth_sun*is_even]){
		spur_gear (modulus, number_of_teeth_sun,missing_teeth_sun, gear_thickness, sun_bore_size,bore_width_of_flats, pressure_angle, -helix_angle, optimized);		// sun gear
	}

	if (built_together){
        if(number_of_planets==0){
            list = [ for (n=[2 : 1 : n_max]) if ((((number_of_teeth_ring_gear+number_of_teeth_sun)/n)==floor((number_of_teeth_ring_gear+number_of_teeth_sun)/n))) n];
            number_of_planets = list[0];										// Determined number of planet gears
             center_distance = modulus*(number_of_teeth_sun + number_of_teeth_planet)/2;		// Distance from sun wheel / ring gear axle
            for(n=[0:1:number_of_planets-1]){
                translate(radius_to_cartesian([center_distance,90,360/number_of_planets*n]))
					rotate([0,0,n*360*d_sun/d_planet])
						spur_gear (modulus, number_of_teeth_planet,missing_teeth_planet,  gear_thickness, planet_bore_size, bore_width_of_flats,pressure_angle, helix_angle);	// planetary gears
            }
       }
       else{
            center_distance = modulus*(number_of_teeth_sun + number_of_teeth_planet)/2;		// Distance from sun wheel / ring gear axis
            for(n=[0:1:number_of_planets-1]){
                translate(radius_to_cartesian([center_distance,90,360/number_of_planets*n]))
                rotate([0,0,n*360*d_sun/(d_planet)])
                    spur_gear (modulus, number_of_teeth_planet,missing_teeth_planet,  gear_thickness, planet_bore_size,bore_width_of_flats, pressure_angle, helix_angle);	// planetary gears
            }
		}
	}
	else{
		planet_distance = number_of_teeth_ring_gear*modulus/2+ring_edge_width+d_planet;		// Distance planets from each other
		for(i=[-(number_of_planets-1):2:(number_of_planets-1)]){
			translate([planet_distance, d_planet*i,0])
				spur_gear (modulus, number_of_teeth_planet,missing_teeth_planet, gear_thickness, planet_bore_size, bore_width_of_flats,pressure_angle, helix_angle);	// planetary gears
		}
	}

	spur_ring_gear (modulus, number_of_teeth_ring_gear,missing_teeth_ring, gear_thickness, ring_edge_width, pressure_angle, helix_angle); // ring_gear

}


//  planetary_herring_bone_gear(modulus=1, number_of_teeth_sun=20, number_of_teeth_planet=10, number_of_planets=6, gear_thickness=5, ring_edge_width=3, sun_bore_size=4,planet_bore_size=2,bore_width_of_flats=4, pressure_angle=20, helix_angle=30, built_together=true, optimized=true);


/*	planetary_herring_bone_gear; uses the modules "herring_bone_spur_gear" and "herring_bone_ring_gear"
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth_sun = Number of teeth of sun gear
    missing_teeth_sun =
    number_of_teeth_planet = Number of teeth of a planetary gear
    missing_teeth_planet =
    missing_teeth_ring =
    Number of planet gears. If null, the function will calculate the minimum number.
    gear_thickness = thickness of gear
	ring_edge_width = width of the edge from root circle
    sun_bore_size = Diameter of the sun gear center bore
    planet_bore_size = Diameter of the planet gears center bore
    bore_width_of_flats=
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle to the rotation axis, default value = 0 ° (straight toothing)
	optimized = Holes for material / weight savings bzw. Create surface enlargement if geometry allows
	built_together = Components assembled for construction or apart for 3D printing	*/

module planetary_herring_bone_gear( modulus,
                                    number_of_teeth_sun,
                                    missing_teeth_sun,
                                    number_of_teeth_planet,
                                    missing_teeth_planet,
                                    missing_teeth_ring,
                                    number_of_planets, 
                                    gear_thickness, 
                                    ring_edge_width, 
                                    sun_bore_size, 
                                    planet_bore_size,
                                    bore_width_of_flats, 
                                    pressure_angle=20, 
                                    helix_angle=0, 
                                    built_together=true, 
                                    optimized=true)
{

	// Dimensions calculations
	d_sun = modulus*number_of_teeth_sun;										// Pitch circle diameter sun
	d_planet = modulus*number_of_teeth_planet;									// Pitch circle planet
	center_distance = modulus*(number_of_teeth_sun +  number_of_teeth_planet) / 2;		// Distance from sun gear / ring_gear axis
	number_of_teeth_ring_gear = number_of_teeth_sun + 2*number_of_teeth_planet;				// Number of teeth of the ring gear
    d_ring_gear = modulus*number_of_teeth_ring_gear;									// Pitch diameter ring gear

	is_even = is_number_even(number_of_teeth_planet);								// Does the sun gear have to be turned?
		
	n_max = floor(180/asin(modulus*(number_of_teeth_planet)/(modulus*(number_of_teeth_sun +  number_of_teeth_planet))));
																		// Number of planet gears: at most as many as without
																		// Overlap possible

	// generate shape
	rotate([0,0,180/number_of_teeth_sun*is_even]){
		herring_bone_spur_gear (modulus, number_of_teeth_sun,missing_teeth_sun, gear_thickness, sun_bore_size,bore_width_of_flats, pressure_angle, -helix_angle, optimized);		// sun gear
	}

	if (built_together){
        if(number_of_planets==0){
            list = [ for (n=[2 : 1 : n_max]) if ((((number_of_teeth_ring_gear+number_of_teeth_sun)/n)==floor((number_of_teeth_ring_gear+number_of_teeth_sun)/n))) n];
            number_of_planets = list[0];										// Determined number of planet gears
             center_distance = modulus*(number_of_teeth_sun + number_of_teeth_planet)/2;		// Distance from sun wheel / ring gear axle
            for(n=[0:1:number_of_planets-1]){
                translate(radius_to_cartesian([center_distance,90,360/number_of_planets*n]))
					rotate([0,0,n*360*d_sun/d_planet])
						herring_bone_spur_gear (modulus, number_of_teeth_planet,missing_teeth_planet, gear_thickness, planet_bore_size,bore_width_of_flats, pressure_angle, helix_angle);	// planetary gears
            }
       }
       else{
            center_distance = modulus*(number_of_teeth_sun + number_of_teeth_planet)/2;		// Distance from sun wheel / ring gear axis
            for(n=[0:1:number_of_planets-1]){
                translate(radius_to_cartesian([center_distance,90,360/number_of_planets*n]))
                rotate([0,0,n*360*d_sun/(d_planet)])
                    herring_bone_spur_gear (modulus, number_of_teeth_planet,missing_teeth_planet, gear_thickness, planet_bore_size,bore_width_of_flats, pressure_angle, helix_angle);	// planetary gears
            }
		}
	}
	else{
		planet_distance = number_of_teeth_ring_gear*modulus/2+ring_edge_width+d_planet;		// Distance planets from each other
		for(i=[-(number_of_planets-1):2:(number_of_planets-1)]){
			translate([planet_distance, d_planet*i,0])
				herring_bone_spur_gear (modulus, number_of_teeth_planet,missing_teeth_planet, gear_thickness, planet_bore_size,bore_width_of_flats, pressure_angle, helix_angle);	// planetary gears
		}
	}

	herring_bone_ring_gear (modulus, number_of_teeth_ring_gear,missing_teeth_ring, gear_thickness, ring_edge_width, pressure_angle, helix_angle); // ring_gear

}

/*  bevel_gear
    modulus =  pitch diameter divided by number of teeth, Specification for the outside of the cone
    number_of_teeth = Number of gear teeth
    missing_teeth =
    bevel_angle = (Half) angle of the cone on which the other ring_gear meshes
    tooth_width = width of the teeth from the outside towards the top of the cone
    bore_size = Diameter of the center bore
    bore_width_of_flats =
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
	helix_angle = Helix angle, default = 0 ° */

module bevel_gear(  modulus, 
                    number_of_teeth,
                    missing_teeth,
                    bevel_angle, 
                    tooth_width, 
                    bore_size,
                    bore_width_of_flats,
                    pressure_angle = 20, 
                    helix_angle=0) 
{

	// Dimensions calculations
	dia_at_base_of_cone = modulus * number_of_teeth;		    // diameter at the base of cone,
                                                                // corresponds to the chord in a spherical section
	radius_at_base_of_cone = dia_at_base_of_cone / 2;		    // diameter at the base of cone,
	rg_Outside = radius_at_base_of_cone/sin(bevel_angle);	    // Large bevel_gearius for tooth outside, corresponds to the length of cone flank;
	rg_Inside = rg_Outside - tooth_width;					    // Big bevel_gearius for tooth inside
	r_Inside = radius_at_base_of_cone*rg_Inside/rg_Outside;
	alpha_angle = atan(tan(pressure_angle)/cos(helix_angle));   // Helix angle in the frontal section
	delta_b = asin(cos(alpha_angle)*sin(bevel_angle));		    // Basic cone angle	
	tip_circle_dia_Outside = (modulus <1)? dia_at_base_of_cone + (modulus * 2.2) * cos(bevel_angle): dia_at_base_of_cone + modulus * 2 * cos(bevel_angle);
	tip_circle_radius_Outside = tip_circle_dia_Outside / 2;
	delta_a = asin(tip_circle_radius_Outside/rg_Outside);
	tip_clearance = modulus / 6;							// TIP clearance
	root_circle_dia_Outside = dia_at_base_of_cone - (modulus +tip_clearance) * 2 * cos(bevel_angle);
	root_circle_radius_Outside = root_circle_dia_Outside / 2;
	delta_f = asin(root_circle_radius_Outside/rg_Outside);
	rkf = rg_Outside*sin(delta_f);									// Radius of the cone foot
	height_f = rg_Outside*cos(delta_f);								// Height of the cone from the foot cone
	
	echo("Part cone diameter on the cone base = ", dia_at_base_of_cone);
	
	// Sizes for complementary truncated cone
	height_k = (rg_Outside-tooth_width)/cos(bevel_angle);			// Height of the complementary cone for correct tooth length
	rk = (rg_Outside-tooth_width)/sin(bevel_angle);				// Foot radius of the complementary cone
	root_circle_radiusk = rk*height_k*tan(delta_f)/(rk+height_k*tan(delta_f));		// tip radius of the cylinder for
																	// Complementary truncated cone
	height_fk = rk*height_k/(height_k*tan(delta_f)+rk);				// height of the complementary truncated cone

	echo("Height bevel_gear = ", height_f-height_fk);
	
	angle_to_involute_point = curve_inverlute(delta_b, bevel_angle);// Angle to the point of the involute on partial cone
		
	// Torsion angle gear_skew_angle from helix angle
	gamma_g = 2*atan(tooth_width*tan(helix_angle)/(2*rg_Outside-tooth_width));
	gear_skew_angle = 2*asin(rg_Outside/radius_at_base_of_cone*sin(gamma_g/2));
	
	step = (delta_a - delta_b)/16;
	circ_pitch = 360/number_of_teeth;												// pitch angle
	start = (delta_b > delta_f) ? delta_b : delta_f;
	mirror_point = (180*(1-clearance))/number_of_teeth+2*angle_to_involute_point;

	// generate shape
	rotate([0,0,angle_to_involute_point+90*(1-clearance)/number_of_teeth]){		// Center the tooth on the x-axis;
                                                                                // makes alignment with other wheels easier
		translate([0,0,height_f]) rotate(a=[0,180,0]){
			union()
            {
				translate([0,0,height_f]) rotate(a=[0,180,0])
                {								// truncated cone							
					difference()
                    {
						linear_extrude(height=height_f-height_fk, scale=root_circle_radiusk/rkf) circle(rkf*1.001); // 1 per mil overlap with tooth root
						translate([0,0,-1]){
                            
							//cylinder(h = height_f-height_fk+2, r = bore_size/2);	// bore_size (was original code )
                            
                            // the next bit added to put flats on side of bore
                            // if no flats required make flats parameter bigger than bore parameter                


                            linear_extrude(height = height_f-height_fk+2,twist=0)
                            { 

                                difference()
                                { 
                                    //draw a 2D circle on centre axis
                                    circle(d=bore_size);        
                                    //draw a square and deduct from circle to blank one side
                                    translate([-bore_width_of_flats*1.5,-bore_size/2]) 
                                    square([bore_width_of_flats,bore_size]);
                                    //draw a square and deduct from circle to blank other side
                                    translate([bore_width_of_flats/2,-bore_size/2]) 
                                    square([bore_width_of_flats,bore_size]);
                                }
                            // end of flats coding

                            }
                            
						}
					}	
				}
				for (rot = [0:circ_pitch:360-((missing_teeth+1)*circ_pitch)]){
					rotate (rot) {															// "number_of_teeth times" copy and rotate
						union(){
							if (delta_b > delta_f){
								// tooth root
								flank_point_below = 1*mirror_point;
								flank_point_above = curve_inverlute(delta_f, start);
								polyhedron(
									points = [
										radius_to_cartesian([rg_Outside, start*1.001, flank_point_below]),	// 1 per mil overlap with tooth
										radius_to_cartesian([rg_Inside, start*1.001, flank_point_below+gear_skew_angle]),
										radius_to_cartesian([rg_Inside, start*1.001, mirror_point-flank_point_below+gear_skew_angle]),
										radius_to_cartesian([rg_Outside, start*1.001, mirror_point-flank_point_below]),								
										radius_to_cartesian([rg_Outside, delta_f, flank_point_below]),
										radius_to_cartesian([rg_Inside, delta_f, flank_point_below+gear_skew_angle]),
										radius_to_cartesian([rg_Inside, delta_f, mirror_point-flank_point_below+gear_skew_angle]),
										radius_to_cartesian([rg_Outside, delta_f, mirror_point-flank_point_below])								
									],
									faces = [[0,1,2],[0,2,3],[0,4,1],[1,4,5],[1,5,2],[2,5,6],[2,6,3],[3,6,7],[0,3,7],[0,7,4],[4,6,5],[4,7,6]],
									convexity =1
								);
							}
							// tooth
							for (delta = [start:step:delta_a-step]){
								flank_point_below = curve_inverlute(delta_b, delta);
								flank_point_above = curve_inverlute(delta_b, delta+step);
								polyhedron(
									points = [
										radius_to_cartesian([rg_Outside, delta, flank_point_below]),
										radius_to_cartesian([rg_Inside, delta, flank_point_below+gear_skew_angle]),
										radius_to_cartesian([rg_Inside, delta, mirror_point-flank_point_below+gear_skew_angle]),
										radius_to_cartesian([rg_Outside, delta, mirror_point-flank_point_below]),								
										radius_to_cartesian([rg_Outside, delta+step, flank_point_above]),
										radius_to_cartesian([rg_Inside, delta+step, flank_point_above+gear_skew_angle]),
										radius_to_cartesian([rg_Inside, delta+step, mirror_point-flank_point_above+gear_skew_angle]),
										radius_to_cartesian([rg_Outside, delta+step, mirror_point-flank_point_above])									
									],
									faces = [[0,1,2],[0,2,3],[0,4,1],[1,4,5],[1,5,2],[2,5,6],[2,6,3],[3,6,7],[0,3,7],[0,7,4],[4,6,5],[4,7,6]],
									convexity =1
								);
							}
						}
					}
				}	
			}
		}
	}
}

/*  herring_bone_bevel_gear; uses the modulue "bevel_gear"
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth = Number of gear teeth
    missing_teeth =
    bevel_angle, tooth_width
    bore_size = Diameter of the center bore
    bore_width_of_flats =
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle, default = 0 °*/

module herring_bone_bevel_gear( modulus, 
                                number_of_teeth,
                                missing_teeth,
                                bevel_angle, 
                                tooth_width, 
                                bore_size,
                                bore_width_of_flats,
                                pressure_angle = 20, 
                                helix_angle=0)
{

	// Dimensions calculations
	
	tooth_width = tooth_width / 2;
	
	dia_at_base_of_cone = modulus * number_of_teeth;								// Part cone diameter on the cone base,
																// corresponds to the chord in a spherical section
	radius_at_base_of_cone = dia_at_base_of_cone / 2;									// Part bevel_gearius on the cone base
	rg_Outside = radius_at_base_of_cone/sin(bevel_angle);					// Great bevel_gearius, corresponds to the length of cone flank;
	tip_clearance = modulus / 6;												// TIP clearance
	root_circle_dia_Outside = dia_at_base_of_cone - (modulus +tip_clearance) * 2 * cos(bevel_angle);
	root_circle_radius_Outside = root_circle_dia_Outside / 2;
	delta_f = asin(root_circle_radius_Outside/rg_Outside);
	height_f = rg_Outside*cos(delta_f);							// Height of the cone from the root cone
	// Twist angle for linear extrusion
	gamma_g = 2*atan(tooth_width*tan(helix_angle)/(2*rg_Outside-tooth_width));
	gear_skew_angle = 2*asin(rg_Outside/radius_at_base_of_cone*sin(gamma_g/2));
	
	echo("Part cone diameter on the cone base= ", dia_at_base_of_cone);
	
	// Sizes for complementary truncated cone
	height_k = (rg_Outside-tooth_width)/cos(bevel_angle);		// Height of the complementary cone for correct tooth length
	rk = (rg_Outside-tooth_width)/sin(bevel_angle);			// root radius of the complementary cone
	root_circle_radiusk = rk*height_k*tan(delta_f)/(rk+height_k*tan(delta_f));	// tip radius of the cylinder for
																// Complementary truncated cone
	height_fk = rk*height_k/(height_k*tan(delta_f)+rk);			// height of the complementary truncated cone
	
	modulus_Inside = modulus*(1-tooth_width/rg_Outside);

		union(){
		bevel_gear(modulus, number_of_teeth, missing_teeth, bevel_angle, tooth_width, bore_size,bore_width_of_flats,, pressure_angle, helix_angle);		// bottom half
		translate([0,0,height_f-height_fk])
		rotate(a=-gear_skew_angle,v=[0,0,1])
		bevel_gear(modulus_Inside, number_of_teeth,missing_teeth, bevel_angle, tooth_width, bore_size,bore_width_of_flats, pressure_angle, -helix_angle);	// upper half
	}
}

/*  Spiral_bevel_gear; uses the modulue "bevel_gear"
    modulus =  pitch diameter divided by number of teeth
    number_of_teeth = Number of gear teeth
    missing_teeth =
    height = Height of the gear
    bore_size = Diameter of the center bore
    bore_width_of_flats =
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle, default = 0 ° */

module spiral_bevel_gear(   modulus,
                            number_of_teeth,
                            missing_teeth,
                            bevel_angle, 
                            tooth_width, 
                            bore_size,
                            bore_width_of_flats,
                            pressure_angle = 20, 
                            helix_angle=30)
{

	steps = 20;

	// Dimensions calculations
	
	b = tooth_width / steps;	
	dia_at_base_of_cone = modulus * number_of_teeth;								// Part cone diameter on the cone base,
																// corresponds to the chord in a spherical section
	radius_at_base_of_cone = dia_at_base_of_cone / 2;									// Part bevel_gearius on the cone base
	rg_Outside = radius_at_base_of_cone/sin(bevel_angle);					// Great bevel_gearius, corresponds to the length of cone flank;
	rg_midpoint = rg_Outside-tooth_width/2;

	echo("Part cone diameter on the cone base = ", dia_at_base_of_cone);

	a=tan(helix_angle)/rg_midpoint;
	
	union(){
	for(i=[0:1:steps-1]){
		r = rg_Outside-i*b;
		helix_angle = a*r;
		modulus_r = modulus-b*i/rg_Outside;
		translate([0,0,b*cos(bevel_angle)*i])
			
			rotate(a=-helix_angle*i,v=[0,0,1])
				bevel_gear(modulus_r, number_of_teeth,missing_teeth, bevel_angle, b, bore_size, bore_width_of_flats, pressure_angle, helix_angle);
		}
	}
}

/*	bevel_gear_pair :with any angle_of_axis; uses the modulue "bevel_gear"
    modulus =  pitch diameter divided by number of teeth, Specification for the outside of the cone
    number_of_teeth_on_gear = Number of gear teeth on gear
    missing_teeth_gear =
    number_of_teeth_pinion = Number of gear teeth on pinion
    missing_teeth_pinion =
	angle_of_axis = Angle between the axles of wheel and pinion
    tooth_width = width of the teeth from the outside towards the top of the cone
    bore_size_on_gear = Diameter of the center bore on the gear
    bore_width_of_flats_gear =
    bore_size_pinion = Diameter of the center bore on the pinion
    bore_width_of_flats_pinion =
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
	helix_angle = Helix angle, default = 0 °
	built_together = Components assembled for construction or apart for 3D printing*/

module bevel_gear_pair( modulus, 
                        number_of_teeth_on_gear,
                        missing_teeth_gear,
                        number_of_teeth_pinion,
                        missing_teeth_pinion, 
                        angle_of_axis=90, 
                        tooth_width, 
                        bore_size_on_gear,
                        bore_width_of_flats_gear,
                        bore_size_pinion,
                        bore_width_of_flats_pinion,
                        pressure_angle=20, 
                        helix_angle=0, 
                        built_together=true)
{
 
	// Dimensions calculations
	r_rad = modulus*number_of_teeth_on_gear/2;							// Part bevel_gearius of the wheel
	delta_rad = atan(sin(angle_of_axis)/(number_of_teeth_pinion/number_of_teeth_on_gear+cos(angle_of_axis)));	// Cone angle of the wheel
	delta_pinion = atan(sin(angle_of_axis)/(number_of_teeth_on_gear/number_of_teeth_pinion+cos(angle_of_axis)));// Cone angle of the pinion
	rg = r_rad/sin(delta_rad);								// Radius of the big ball
	tip_clearance = modulus / 6;											// TIP clearance
	root_circle_dia_pinion = pi*rg*delta_pinion/90 - 2 * (modulus + tip_clearance);	// Foot cone diameter on the big ball
	root_circle_radius_pinion = root_circle_dia_pinion / 2;								// Foot bevel_gearius on the big ball
	delta_f_pinion = root_circle_radius_pinion/(pi*rg) * 180;				// Tip angle
	rkf_pinion = rg*sin(delta_f_pinion);					// Radius of the cone foot
	height_f_pinion = rg*cos(delta_f_pinion);				// Höhe des Kegels vom Fußkegel
	
	echo("\bevel angle gear = ", delta_rad);
	echo("bevel angle pinion = ", delta_pinion);
 
	root_circle_dia_rad = pi*rg*delta_rad/90 - 2 * (modulus + tip_clearance);			// Foot cone diameter on the big ball
	root_circle_radius_rad = root_circle_dia_rad / 2;									// Fussbevel_gearius on the big ball
	delta_f_rad = root_circle_radius_rad/(pi*rg) * 180;						// Tip angle
	rkf_rad = rg*sin(delta_f_rad);							// Radius of the cone foot
	height_f_rad = rg*cos(delta_f_rad);						// Height of the cone from the foot cone

	echo("Height gear = ", height_f_rad);
	echo("Height pinion = ", height_f_pinion);
	
	is_even = is_number_even(number_of_teeth_pinion);
	
	// generate shape
	// gear
	rotate([0,0,180*(1-clearance)/number_of_teeth_on_gear*is_even])
		bevel_gear(modulus, number_of_teeth_on_gear,missing_teeth_gear, delta_rad, tooth_width, bore_size_on_gear,bore_width_of_flats_gear, pressure_angle, helix_angle);
	
	// pinion
	if (built_together)
		translate([-height_f_pinion*cos(90-angle_of_axis),0,height_f_rad-height_f_pinion*sin(90-angle_of_axis)])
			rotate([0,angle_of_axis,0])
				bevel_gear(modulus, number_of_teeth_pinion,missing_teeth_pinion, delta_pinion, tooth_width, bore_size_pinion,bore_width_of_flats_pinion, pressure_angle, -helix_angle);
	else
		translate([rkf_pinion*2+modulus+rkf_rad,0,0])
			bevel_gear(modulus, number_of_teeth_pinion,missing_teeth_pinion, delta_pinion, tooth_width, bore_size_pinion, pressure_angle, -helix_angle);
 }

/*	herring_bone_bevel_gear_pair with any angle_of_axis; uses the modulue "herring_bone_bevel_gear"
    modulus =  pitch diameter divided by number of teeth, Specification for the outside of the cone
    number_of_teeth_on_gear = Number of gear teeth on gear
    missing_teeth_gear =
    number_of_teeth_pinion = Number of gear teeth on pinion
    missing_teeth_pinion =
	angle_of_axis = Angle between the axles of wheel and pinion
    tooth_width = width of the teeth from the outside towards the top of the cone
    bore_size_on_gear = Diameter of the center bore on the gear
    bore_width_of_flats_gear,
    bore_size_pinion = Diameter of the center bore on the pinion
    bore_width_of_flats_pinion,
    pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
    helix_angle = Helix angle, default = 0°
	built_together = Components assembled for construction or apart for 3D printing */
 
module herring_bone_bevel_gear_pair(modulus,
                                    number_of_teeth_on_gear,
                                    missing_teeth_gear,
                                    number_of_teeth_pinion,
                                    missing_teeth_pinion, 
                                    angle_of_axis=90, 
                                    tooth_width, 
                                    bore_size_on_gear,
                                    bore_width_of_flats_gear,
                                    bore_size_pinion,
                                    bore_width_of_flats_pinion, 
                                    pressure_angle = 20, 
                                    helix_angle=10, 
                                    built_together)
{
 
	r_rad = modulus*number_of_teeth_on_gear/2;							// Part bevel_gearius of the wheel
	delta_rad = atan(sin(angle_of_axis)/(number_of_teeth_pinion/number_of_teeth_on_gear+cos(angle_of_axis)));	// Cone angle of the wheel
	delta_pinion = atan(sin(angle_of_axis)/(number_of_teeth_on_gear/number_of_teeth_pinion+cos(angle_of_axis)));// Cone angle of the pinion
	rg = r_rad/sin(delta_rad);								// Radius of the big ball
	tip_clearance = modulus / 6;											// TIP clearance
	root_circle_dia_pinion = pi*rg*delta_pinion/90 - 2 * (modulus + tip_clearance);	// Foot cone diameter on the big ball
	root_circle_radius_pinion = root_circle_dia_pinion / 2;								// Fussbevel_gearius on the big ball
	delta_f_pinion = root_circle_radius_pinion/(pi*rg) * 180;				// Tip angle
	rkf_pinion = rg*sin(delta_f_pinion);					// Radius of the cone foot
	height_f_pinion = rg*cos(delta_f_pinion);				// Height of the cone from the foot cone
	
	echo("Bevel angle wheel = ", delta_rad);
	echo("Bevel angle pinion = ", delta_pinion);
 
	root_circle_dia_rad = pi*rg*delta_rad/90 - 2 * (modulus + tip_clearance);			// Foot cone diameter on the big ball
	root_circle_radius_rad = root_circle_dia_rad / 2;									// Foot bevel_gearius on the big ball
	delta_f_rad = root_circle_radius_rad/(pi*rg) * 180;						// Tip angle
	rkf_rad = rg*sin(delta_f_rad);							// Radius of the cone foot
	height_f_rad = rg*cos(delta_f_rad);						// Height of the cone from the foot cone

	echo("Height gear = ", height_f_rad);
	echo("Height pinion = ", height_f_pinion);
	
	is_even = is_number_even(number_of_teeth_pinion);
	
	// gear
	rotate([0,0,180*(1-clearance)/number_of_teeth_on_gear*is_even])
		herring_bone_bevel_gear(modulus, number_of_teeth_on_gear,missing_teeth_gear, delta_rad, tooth_width, bore_size_on_gear,bore_width_of_flats_gear, pressure_angle, helix_angle);
	
	// pinion
	if (built_together)
		translate([-height_f_pinion*cos(90-angle_of_axis),0,height_f_rad-height_f_pinion*sin(90-angle_of_axis)])
			rotate([0,angle_of_axis,0])
				herring_bone_bevel_gear(modulus, number_of_teeth_pinion,missing_teeth_pinion, delta_pinion, tooth_width, bore_size_pinion, bore_width_of_flats_pinion, pressure_angle, -helix_angle);
	else
		translate([rkf_pinion*2+modulus+rkf_rad,0,0])
			herring_bone_bevel_gear(modulus, number_of_teeth_pinion,missing_teeth_pinion, delta_pinion, tooth_width, bore_size_pinion,bore_width_of_flats_pinion, pressure_angle, -helix_angle);

}

/*
Calculates a worm / Archimedean screw.
modulus =  pitch diameter divided by number of teeth, Height of the worm tip over the partial cylinder
number_of_starts = Number of gears (teeth) of the worm
worm_length = Length of worm
bore_size = Diameter of the center bore
pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
gradient_angle = Slope angle of the worm, corresponds to 90 ° minus helix angle. Positive slope angle = clockwise.
unsplit_worm_print=splits the worm for printing to give stength




*/


module worm(modulus, 
            number_of_starts, 
            worm_length, 
            bore_size,
            bore_width_of_flats,
            pressure_angle=20, 
            gradient_angle,
            unsplit_worm_print)
{

	// Dimensions calculations
	tip_clearance = modulus / 6;												// TIP clearance
	pitch_circle_radius = modulus*number_of_starts/(2*sin(gradient_angle));		// partial_cylinder-Radius
	root_circle_radius = pitch_circle_radius - modulus - tip_clearance;											// Foot cylinder radius
	a = modulus*number_of_starts/(90*tan(pressure_angle));				// spiral parameters
	circ_pitch_max = 180/number_of_starts*tan(pressure_angle);				// Angle from foot to tip in the normal
	gear_skew_angle = -rad*worm_length/((root_circle_radius+modulus+tip_clearance)*tan(gradient_angle));	// Torsion angle for extrusion
	
	step = circ_pitch_max/16;
    
    
	// generate shape: extrude with twist a surface enclosed by two Archimedean spirals
    if (unsplit_worm_print) {

difference()
    {  
        union()
        {//draw shape




		rotate([0,0,circ_pitch_max]){//aligns worm and gear when together
			linear_extrude(height = worm_length, center = false, convexity = 10, twist = gear_skew_angle){
				difference(){
					union(){
						for(i=[0:1:number_of_starts-1]){
							polygon(
								concat(							
									[[0,0]],
									
									// rising tooth flank
									[for (circ_pitch = [0:step:circ_pitch_max])
										polar_to_cartesian([spiral(a, root_circle_radius, circ_pitch), circ_pitch+i*(360/number_of_starts)])],
										
									// Tooth tip
									[for (circ_pitch = [circ_pitch_max:step:180/number_of_starts])
										polar_to_cartesian([spiral(a, root_circle_radius, circ_pitch_max), circ_pitch+i*(360/number_of_starts)])],
									
									// descending tooth flank
									[for (circ_pitch = [180/number_of_starts:step:(180/number_of_starts+circ_pitch_max)])
										polar_to_cartesian([spiral(a, root_circle_radius, 180/number_of_starts+circ_pitch_max-circ_pitch), circ_pitch+i*(360/number_of_starts)])]
								)
							);
						}
						circle(root_circle_radius);
					}
               
				}   //end difference
			}       //end linear exfrude
           
		}
 
 
 /*maybe add fixxing bosses
        
        //upper fixing boss
        translate([0,0,worm_length+5])
        cylinder(10,13,13,true);
        //lower fixing boss 
        translate([0,0,-5])
        cylinder(10,pitch_circle_radius,pitch_circle_radius,true); 
 */
    }
               // the next bit added to put flats on side of bore
               // if no flats required make flats parameter bigger than bore parameter                
   			
                    linear_extrude(height = worm_length, center = false)
                    {  
                        difference()
                        {
                            //draw a 2D circle on centre axis
                            circle(d=bore_size);        
                            //draw a square and deduct from circle to blank one side
                            translate([-bore_width_of_flats*1.5,-bore_size/2]) 
                            square([bore_width_of_flats,bore_size]);
                            //draw a square and deduct from circle to blank other side
                            translate([bore_width_of_flats/2,-bore_size/2]) 
                            square([bore_width_of_flats,bore_size]);
                        }
                    }     
    
}
    }       
 else {
        // grnerate two worms and split them in half for printing (gives stronger worm)
		difference(){
			union()
            {
                //generate a worm to split in half
				translate([1,pitch_circle_radius*1.5,0])
                {
					rotate([90,0,90])
					worm(modulus, number_of_starts, worm_length, bore_size,bore_width_of_flats, pressure_angle, gradient_angle, unsplit_worm_print=true);//unsplit_worm_print must be true here
				}
                //generate another worm to split in half
				translate([worm_length+1,-pitch_circle_radius*1.5,0])
                {
					rotate([90,0,-90])
					worm(modulus, number_of_starts, worm_length, bore_size,bore_width_of_flats, pressure_angle, gradient_angle, unsplit_worm_print=true);//unsplit_worm_print must be true here
				}
                
			}
                // generate a box to deduct half worms from 
                translate([worm_length/2+1,0,-(pitch_circle_radius+modulus+1)/2])
                {
					cube([worm_length+22,3*pitch_circle_radius+2*(pitch_circle_radius+modulus+1),pitch_circle_radius+modulus+1], center = true);//22 is 2+ 2 boss lengths
                }
		}
	}       
}

/*
generates a worm and matching spur gear.
modulus =  pitch diameter divided by number of teeth, Height of the worm tip over the partial cylinder or the tooth tip above the pitch circle
number_of_teeth = Number of teeth on gear wheel
missing_teeth =
number_of_starts = Number of gears (teeth) of the worm
width = tooth_width
worm_length = Length of worm
bore_size_worm = Diameter of the center bore on the worm
bore_width_of_flats_worm =
bore_size_on_gear = Diameter of the center bore on the spur gear
bore_width_of_flats_gear =
pressure_angle = pressure_angle, Standard value = 20 ° according to DIN 867. Should not be greater than 45 °.
gradient_angle = Lead angle of the worm, corresponds to 90 ° bevel angle. Positive slope angle = clockwise.
optimized = Holes for material / weight savings
built_together =  Components assembled for construction or apart for 3D printing */

module worm_and_spur_gear(  modulus, 
                            number_of_teeth,
                            missing_teeth,
                            number_of_starts, 
                            width, 
                            worm_length, 
                            bore_size_worm,
                            bore_width_of_flats_worm, 
                            bore_size_gear,
                            bore_width_of_flats_gear, 
                            pressure_angle=20, 
                            gradient_angle, 
                            optimized,
                            built_together,
                            unsplit_worm_print)
{
	
	tip_clearance = modulus / 6;												// TIP clearance
	r_worm = modulus*number_of_starts/(2*sin(gradient_angle));		// partial_cylinder-Radius worm
	r_rad = modulus*number_of_teeth/2;									// Part bevel_gearius spur_gear
	root_circle_radius_worm = r_worm - modulus - tip_clearance;						// Foot cylinder radius
	gear_skew_angle = -90*width*sin(gradient_angle)/(pi*r_rad);			// Rotation angle spur_gear
	zahnabstand = modulus*pi/cos(gradient_angle);				// Teeth spacing in transverse section
	x = is_number_even(number_of_starts)? 0.5 : 1;

	if (built_together) {
		translate([r_worm,(ceil(worm_length/(2*zahnabstand))-x)*zahnabstand,0])
			rotate([90,180/number_of_starts,0])
				worm(modulus, number_of_starts, worm_length, bore_size_worm,bore_width_of_flats_worm, pressure_angle, gradient_angle, unsplit_worm_print);//unsplit_worm_print must be true here

		translate([-r_rad,0,-width/2])
			rotate([0,0,gear_skew_angle])
				spur_gear (modulus, number_of_teeth,missing_teeth, width, bore_size_gear,bore_width_of_flats_gear, pressure_angle, -gradient_angle, optimized);
	}
	else {	
		worm(modulus, number_of_starts, worm_length, bore_size_worm,bore_width_of_flats_worm, pressure_angle, gradient_angle, unsplit_worm_print);

		translate([-2*r_rad,0,0])
			spur_gear (modulus, number_of_teeth, missing_teeth,width, bore_size_gear,bore_width_of_flats_gear, pressure_angle, -gradient_angle, optimized);
	}
}


////////////////////////////////////////////////////////////////////////////
//function to return clearance hole size for a metric thread

 
function ThreadToClearanceHole(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.05],
      [2.5, 2.6],
      [3, 3.1],
      [3.5, 3.6],
      [4, 4.1],
      [5, 5.1],
      [6, 6.1],
      [7, 7.2],
      [8, 8.2],
      [10, 10.2],
      [12, 12.2],
      [14, 14.25],
      [16, 16.25],
      [18, 18.25],
      [20, 20.25],
      [22, 22.25],
      [24, 24.25],
      [27, 27.25],
      [30, 30.5],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 60.75]
        ]) : return;
        
        
////////////////////////////////////////////////////////////////////////////
//function to return cap screw head diameter clearance hole size

 
function ThreadToCapHeadDiameter(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 3.5],
      [2.5, 2.6],
      [3, 5.5],
      [3.5, 3.6],
      [4, 7.0],
      [5, 8.5],
      [6, 10.0],
      [7, 7.2],
      [8, 13.0],
      [10, 16.0],
      [12, 18.0],
      [14, 21.0],
      [16, 24.0],
      [18, 27.0],
      [20, 30.0],
      [22, 33.0],
      [24, 36.0],
      [27, 40.0],
      [30, 45.0],
      [33, 33.5],
      [36, 36.5],
      [39, 39.5],
      [42, 42.5],
      [45, 45.5],
      [48, 48.75],
      [52, 52.75],
      [56, 56.75],
      [60, 60.75],
      [64, 60.75]
        ]) : return;
  
 ////////////////////////////////////////////////////////////////////////////
//function to return cap screw head diameter clearance hole size

 
function ThreadToCapHeadDepth(size) =
  (size <= 64) ?
    lookup(size, [
      [2, 2.0],
      [2.5, 2.5],
      [3, 3.0],
      [3.5, 3.5],
      [4, 4.0],
      [5, 5.0],
      [6, 6.0],
      [7, 7.0],
      [8, 8.0],
      [10, 10.0],
      [12, 12.0],
      [14, 14.0],
      [16, 16.0],
      [18, 18.0],
      [20, 20.0],
      [22, 22.0],
      [24, 24.0],
      [27, 27.0],
      [30, 30.0],
      [33, 33.0],
      [36, 36.0],
      [39, 39.0],
      [42, 42.0],
      [45, 45.0],
      [48, 48.0],
      [52, 52.0],
      [56, 56.0],
      [60, 60.0],
      [64, 60.0]
        ]) : return;
  
 